@@ -7,13 +7,14 @@ import pytest
 from box import Box
 
 from framework.exit_code import ExitCode
-from framework.db.mysql_db import MysqlDB
 from framework.db.redis_db import RedisDB
 from framework.utils.log_util import logger
+from framework.render_data import RenderData
 from framework.http_client import ResponseUtil
 from framework.utils.common import snake_to_pascal
 from framework.global_attribute import CONFIG, GlobalAttribute, _FRAMEWORK_CONTEXT
 from config.settings import UNAUTHORIZED_CODE
+
 module = importlib.import_module("test_case.conftest")
 
 
@@ -31,6 +32,8 @@ class BaseTestCase(object):
             app = self.default_app(app)
             app_http = getattr(self.http, app)
             domain = self.context.get(app).get("domain")
+            data = RenderData(data).render()
+            url = data.request.get("url")
             data.request.url = self.replace_domain(data.request.url, domain)
             self.response = getattr(app_http, account).request(data=data, kwargs=kwargs)
             if self.response.status_code in UNAUTHORIZED_CODE:
@@ -38,10 +41,10 @@ class BaseTestCase(object):
                 setattr(_http, app, getattr(module, f"{snake_to_pascal(app)}Login")(app))
                 token_expiry = self.context_get("token_expiry")
                 expire_time = datetime.now() + timedelta(seconds=token_expiry)
-                _FRAMEWORK_CONTEXT.set(key=app, value=expire_time)
+                _FRAMEWORK_CONTEXT.set(app=app, key="expire_time", value=expire_time)
             return self.response
         except AttributeError as e:
-            logger.error(f"app {app} or account {account} no exist: {e}")
+            logger.error(e)
             traceback.print_exc()
             pytest.exit(ExitCode.APP_OR_ACCOUNT_NOT_EXIST)
             return None
@@ -58,9 +61,9 @@ class BaseTestCase(object):
         request.update({"method": "get", "headers": {}, **kwargs})
         return self.request(app=app, account=account, data=Box({"request": request}))
 
-    def put(self, app, account, url, data=None, **kwargs):
+    def put(self, app, account, url, data=None, json=None, **kwargs):
         domain = self.context.get(app).get("domain")
-        request = {"url": urljoin(domain, url), "data": data}
+        request = {"url": urljoin(domain, url), "data": data, "json": json}
         request.update({"method": "put", "headers": {}, **kwargs})
         return self.request(app=app, account=account, data=Box({"request": request}))
 
@@ -72,7 +75,7 @@ class BaseTestCase(object):
 
     def mysql_conn(self, db, app=None):
         try:
-            return MysqlDB(**CONFIG.get(app=self.default_app(app), key="mysql").get(db))
+            return _FRAMEWORK_CONTEXT.get(app=self.default_app(app), key="mysql").get(db)
         except AttributeError as e:
             traceback.print_exc()
             pytest.exit(ExitCode.LOAD_DATABASE_INFO_ERROR)
