@@ -9,7 +9,7 @@ from itertools import chain
 from urllib.parse import urljoin
 from collections import OrderedDict
 from datetime import datetime, timedelta
-
+from itertools import groupby
 import dill
 import retry
 import allure
@@ -17,6 +17,8 @@ import pytest
 from box import Box
 
 from framework.exit_code import ExitCode
+from framework.db.mysql_db import MysqlDB
+from framework.db.redis_db import RedisDB
 from framework.utils.log_util import logger
 from framework.render_data import RenderData
 from framework.utils.yaml_util import YamlUtil
@@ -69,6 +71,7 @@ def pytest_configure(config):
         CONTEXT.set_from_yaml(f"config/{app}/context.yaml", CONTEXT.env, app)
         # 将所有app对应环境的中间件配置加到全局
         CONFIG.set_from_yaml(f"config/{app}/config.yaml", CONTEXT.env, app)
+
     CONTEXT.set(key="all_app", value=all_app)
     sys.path.append(CASES_DIR)
 
@@ -88,8 +91,7 @@ def pytest_generate_tests(metafunc):
     func_file_path = metafunc.module.__file__
     # 获取当前待执行用例的函数名
     func_name = metafunc.function.__name__
-    if func_name in ["test_setup", "test_teardown"]:
-        return
+
     # 获取测试用例所属app
     belong_app = Path(func_file_path).relative_to(CASES_DIR).parts[0]
     # 获取当前用例对应的测试数据路径
@@ -103,14 +105,19 @@ def pytest_generate_tests(metafunc):
     # 测试用例公共数据
     case_common = test_data.get("case_common")
     if case_common.get("ignore"):
-        pytest.skip("Skipp the case because ignore")
-    scenarios = case_common.get("scenarios")
+        return
+
+    # UPDATE: 为了支持Setup和Teardown的分组
+    if func_name in ["test_setup", "test_teardown"]:
+        return
+
     # 测试用例数据
-    case_data = test_data.get(func_name)
+    case_data = test_data.get(func_name) or dict()
     if not case_data:
-        logger.error(f"测试方法: {func_name} 对应的测试数据不存在")
-        traceback.print_exc()
-        pytest.exit(ExitCode.CASE_DATA_NOT_EXIST)
+        case_data["_scenario"] = {"data": {}}
+        case_data["_belong_app"] = belong_app
+        metafunc.parametrize("data", [case_data, ], ids=[f'{case_data.get("title", "")}#'], scope="function")
+        return
     if case_data.get("request") is None:
         case_data["request"] = dict()
     if case_data.get("request").get("headers") is None:
@@ -126,13 +133,10 @@ def pytest_generate_tests(metafunc):
     url = case_data.get("request").get("url")
     method = case_data.get("request").get("method")
     if not url:
+        # UPDATE: 有的步骤不需要请求接口 比如 只校验数据库， 去掉下面的校验
         if not case_common.get("url"):
-            logger.error(f"测试数据request中缺少必填字段: url", case_data)
-            pytest.exit(ExitCode.YAML_MISSING_FIELDS)
-        if case_common.get("url").strip().startswith("${"):
-            case_data["request"]["url"] = case_common.get("url")
-        else:
-            case_data["request"]["url"] = urljoin(domain, case_common.get("url"))
+            logger.warning(f"{func_file_path} request中缺少必填字段: url", case_data)
+            # pytest.exit(ExitCode.YAML_MISSING_FIELDS)
     else:
         if url.strip().startswith("${"):
             case_data["request"]["url"] = url
@@ -140,21 +144,22 @@ def pytest_generate_tests(metafunc):
             case_data["request"]["url"] = urljoin(domain, url)
 
     if not method:
+        # UPDATE: 有的步骤不需要请求接口 比如 只校验数据库， 去掉下面的校验
         if not case_common.get("method"):
-            logger.error(f"测试数据request中缺少必填字段: method", case_data)
-            pytest.exit(ExitCode.YAML_MISSING_FIELDS)
-        case_data["request"]["method"] = case_common.get("method")
+            logger.warning(f"{func_file_path} request中缺少必填字段: method", case_data)
+            # pytest.exit(ExitCode.YAML_MISSING_FIELDS)
 
     for key in ["title", "level"]:
         if key not in case_data:
-            logger.error(f"测试数据{func_name}中缺少必填字段: {key}", case_data)
-            pytest.exit(ExitCode.YAML_MISSING_FIELDS)
+            logger.warning(f"{func_file_path} 缺少必填字段: {key}", case_data)
+            # pytest.exit(ExitCode.YAML_MISSING_FIELDS)
 
     if case_data.get("mark"):
         metafunc.function.marks = [case_data.get("mark"), case_data.get("level")]
     else:
         metafunc.function.marks = [case_data.get("level")]
 
+    scenarios = case_common.get("scenarios")
     case_data_list = list()
     if scenarios:
         ids = list()
@@ -185,6 +190,11 @@ def pytest_generate_tests(metafunc):
 
 
 def pytest_collection_modifyitems(items):
+    # 重新排序
+    new_items = sort(items)
+    # Demo: new_items = [items[0],items[2],items[1],items[3]]
+    items[:] = new_items
+
     for item in items:
         try:
             marks = item.function.marks
@@ -198,6 +208,105 @@ def pytest_collection_modifyitems(items):
             pass
 
 
+# UPDATE 将用例 按类名进行分组的核心方法
+def filtered_groupby(iterable, key_func):
+    """生成器：过滤key_func返回None的元素，并按key_func分组"""
+    current_key = None
+    current_group = []
+
+    for item in iterable:
+        key = key_func(item)
+        if key is None:
+            continue  # 跳过key为None的元素
+
+        # 处理分组逻辑（类似groupby）
+        if key != current_key:
+            if current_group:
+                yield current_key, current_group
+            current_key = key
+            current_group = [item]
+        else:
+            current_group.append(item)
+
+    # 输出最后一组
+    if current_group:
+        yield current_key, current_group
+
+
+# UPDATE：按类的全路径 进行用例分组 同一个类的test方法分到一组
+def __get_group_key__(item):
+    return '::'.join(item.nodeid.split('::')[:2])
+
+
+# UPDATE：改变pytest的原始排序规则
+def sort(case_items):
+    # 按测试类全路径分类,同一个类文件的用例归集到一起
+    # 使用 groupby 函数进行分组
+    item_group_list = [list(group) for _, group in filtered_groupby(case_items, __get_group_key__)]
+
+    all_item_list = []
+    clase_id = None
+    for items in item_group_list:
+        # 未被test_setup/test_teardown 标记的test方法
+        non_custom_scope_items = [item for item in items if
+                                  'test_setup' != item.originalname and 'test_teardown' != item.originalname]
+        item_list = []
+        # 用例的组数
+        case_suite_num = 0
+        # 生成每个组当前的索引
+        ori_name_temp = None
+        ori_name_list = []
+
+        for item in non_custom_scope_items:
+            clase_id = item.cls.__name__
+            original_name = item.originalname
+
+            if ori_name_temp is None or ori_name_temp == original_name:
+                ori_name_temp = original_name
+                case_suite_num += 1
+                ori_name_list.append([original_name, item])
+            else:
+                break
+
+        # 根据组数 创建各组的数组 并插入第一个case
+        case_dict = dict()
+        try:
+            for i in range(case_suite_num):
+                item = ori_name_list[i][1]
+                id = item.callspec.id
+
+                first_part = id.split('#', 1)[-1]
+                index = first_part.split(']')[0]
+                case_dict[index] = [item]
+        except:
+            pass
+
+        new_start_index = case_suite_num
+        # 以new_start_index为起点 重新遍历items
+        try:
+            for i in range(new_start_index, len(non_custom_scope_items)):
+                item = non_custom_scope_items[i]
+                id = item.callspec.id
+                first_part = id.split('#', 1)[-1]
+                index = first_part.split(']')[0]
+                case_dict.get(index).append(item)
+        except:
+            pass
+
+        index = 0
+        for id in case_dict:
+            index += 1
+            case_item_list = case_dict.get(id)
+            for item in case_item_list:
+                allure_suite_mark = f'{clase_id}#{index}'
+                setattr(item, 'allure_suite_mark', allure_suite_mark)
+            item_list += case_item_list
+
+        all_item_list += item_list
+
+    return all_item_list
+
+
 def pytest_collection_finish(session):
     """获取最终排序后的 items 列表"""
     # 过滤掉item名称是test_setup或test_teardown的
@@ -240,7 +349,9 @@ def pytest_runtestloop(session):
     _FRAMEWORK_CONTEXT.set(key="_http", value=login())
 
 
+@pytest.hookimpl
 def pytest_runtest_setup(item):
+    allure.dynamic.sub_suite(item.allure_suite_mark)
     if item.funcargs.get("first"):
         test_object = item.instance
         test_object.context = CONTEXT
@@ -250,15 +361,15 @@ def pytest_runtest_setup(item):
         test_object.data = Box(data)
         test_object.scenario = Box(convert_numbers_to_decimal(data.get("_scenario").get("data")))
         test_object.belong_app = data.get("_belong_app")
-        test_setup = getattr(test_object, "test_setup", None)
-        if test_setup:
+        test_before_scenario = getattr(test_object, "test_setup", None)
+        if test_before_scenario:
             try:
-                test_setup()
+                test_before_scenario()
                 item.funcargs["setup_success"] = True
             except Exception as e:
                 item.funcargs["setup_success"] = False
                 traceback.print_exc()
-                logger.error(f"{item.location[0]} test_setup方法执行异常:{e}")
+                logger.error(f"{item.location[0]} test_before_scenario方法执行异常:{e}")
 
 
 def pytest_runtest_call(item):
@@ -267,11 +378,12 @@ def pytest_runtest_call(item):
     :param item:
     :return:
     """
+    func_name = item.originalname
     ignore_error_and_continue = item.config.getini("ignore_error_and_continue")
     if ignore_error_and_continue == "false":
         # setup方法执行失败，则主动标记用例执行失败，不会执行用例
         if item.funcargs.get("setup_success") is False:
-            pytest.skip(f"{item.nodeid} test_setup execute error")
+            pytest.skip("test_before_scenario execute error")
         # 判断上一个用例是否执行失败，如果上一个用例执行失败，则主动标记用例执行失败，不会执行用例（解决场景性用例，有一个失败则后续用例判为失败）
         index = item.session.items.index(item)
         current_cls_name = item.parent.name
@@ -284,8 +396,10 @@ def pytest_runtest_call(item):
                         1) == current_turn.group(1):  # 确保是同一个类
                     status = getattr(prev_item, "status", None)  # 访问 status 属性
                     skip_reason = getattr(prev_item, "skip_reason", None)  # 访问 skip_reason 属性
-                    if status == "skipped" and skip_reason in ["the previous method execution skipped",
-                                                               "the previous method execution failed"]:
+                    if status == "skipped" and skip_reason.strip() in [
+                        "the previous method execution skipped",
+                        "the previous method execution failed",
+                        "test_before_scenario execute error"]:
                         pytest.skip("the previous method execution skipped")
                     elif status == "failed":
                         pytest.skip("the previous method execution failed")
@@ -294,13 +408,13 @@ def pytest_runtest_call(item):
     origin_data = item.funcargs.get("data")
     __init_allure(origin_data)
     logger.info(f"执行用例: {item.nodeid}")
-    # 对原始请求数据进行渲染替换
-    rendered_data = RenderData(origin_data).render()
+    # # 对原始请求数据进行渲染替换
+    # rendered_data = RenderData(origin_data).render()
     # 函数式测试用例添加参数data, belong_app
     http = item.funcargs.get("http")
-    item.funcargs["data"] = item.instance.data = Box(rendered_data)
+    item.funcargs["data"] = item.instance.data = Box(origin_data)
     item.funcargs["scenario"] = item.instance.scenario = Box(
-        convert_numbers_to_decimal(rendered_data.get("_scenario").get("data")))
+        convert_numbers_to_decimal(origin_data.get("_scenario").get("data")))
     _belong_app = origin_data.get("_belong_app")
     item.funcargs["belong_app"] = item.instance.belong_app = _belong_app
     item.funcargs["config"] = item.instance.config = CONFIG
@@ -316,7 +430,7 @@ def pytest_runtest_call(item):
     #     pytest.exit(ExitCode.MISSING_ASSERTIONS)
 
     # 判断token是否过期，过期则重新登录
-    expire_time = _FRAMEWORK_CONTEXT.get(_belong_app)
+    expire_time = _FRAMEWORK_CONTEXT.get(app=_belong_app, key="expire_time")
     if expire_time:
         _http = _FRAMEWORK_CONTEXT.get("_http")
         if datetime.now() >= expire_time:
@@ -325,18 +439,18 @@ def pytest_runtest_call(item):
             # 更新记录的过期时间
             token_expiry = CONTEXT.get(_belong_app).get("token_expiry")
             expire_time = datetime.now() + timedelta(seconds=token_expiry)
-            _FRAMEWORK_CONTEXT.set(key=_belong_app, value=expire_time)
+            _FRAMEWORK_CONTEXT.set(app=app, key="expire_time", value=expire_time)
 
 
 def pytest_runtest_teardown(item):
     if item.funcargs.get("last") and getattr(item, "status", None) not in ["skipped", "failed"]:
         test_object = item.instance
-        test_teardown = getattr(test_object, "test_teardown", None)
-        if test_teardown:
+        test_after_scenario = getattr(test_object, "test_teardown", None)
+        if test_after_scenario:
             try:
-                test_teardown()
+                test_after_scenario()
             except Exception as e:
-                pytest.fail(f"the test_teardown method execution error: {e}")
+                pytest.fail(f"the test_after_scenario method execution error: {e}")
 
 
 @pytest.hookimpl(tryfirst=True, hookwrapper=True)
@@ -444,7 +558,10 @@ def login():
             token_expiry = CONTEXT.get(app).get("token_expiry")
             if token_expiry:
                 expire_time = datetime.now() + timedelta(seconds=token_expiry)
-                _FRAMEWORK_CONTEXT.set(key=app, value=expire_time)
+                _FRAMEWORK_CONTEXT.set(app=app, key="expire_time", value=expire_time)
+            mysqls = CONFIG.get(app=app, key="mysql")
+            mysqls_obj = {item: MysqlDB(**mysqls.get(item)) for item in mysqls.keys()}
+            _FRAMEWORK_CONTEXT.set(app=app, key="mysql", value=mysqls_obj)
         logger.info("登录完成".center(80, "*"))
         return Http
     except Exception as e:
